/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.syncclient.blackberry;

import java.util.Hashtable;

import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import javax.microedition.pim.PIM;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;
import net.rim.device.api.system.RadioInfo;
import net.rim.device.api.system.TrackwheelListener;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.UiApplication;

import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.BitmapField;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.PasswordEditField;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.container.MainScreen;

import sync4j.syncclient.blackberry.listener.Sync4jContactListener;
import sync4j.syncclient.blackberry.listener.Sync4jEventListener;

import sync4j.syncclient.spds.SyncManagerFactory;
import sync4j.syncclient.util.StaticDataHelper;

import net.rim.device.api.system.DeviceInfo;


/**
 * <code>SyncClient</code> provides with the user interface
 * for synchronizing contact base with SyncServer.
 *
 * @author Santosh email: info@telibrahma.com & Lavanya Nadimpalli email:lavanya@telibrahma.com
 * @author Fabio Maggi @ Funambol
 * $Id: SyncClient.java,v 1.0
 */

public class SyncClient
extends UiApplication
implements Runnable, TrackwheelListener{

    //----------------------------private Constants

    final private static String PROP_SERVER_URL          = "serverUrl"           ;
    final private static String PROP_LOGIN               = "login"               ;
    final private static String PROP_GATEWAYAPN          = "gatewayApn"          ;
    final private static String PROP_GATEWAYIP           = "gatewayIp"           ;
    final private static String PROP_GATEWAYPORT         = "gatewayPort"         ;

    final private static String ERROR_NOT_AUTHORIZED     = "401"                 ;
    final private static String ERROR_NOT_FOUND          = "404"                 ;

    final private static String IMMAGE_LOGO              = "arrows_sync.gif"     ;

    final private static String CONTACT_SORCE_TYPE       = "text/x-s4j-contact"  ;
    final private static String CONTACT_SOURCE_NAME      = "xcard"               ;
    final private static String CALENDAR_SOURCE_TYPE     = "text/x-s4j-calendar" ;
    final private static String CALENDAR_SOURCE_NAME     = "xcal"                ;

    //-------------------------------------------------------------- Constants

    private static PersistentObject store;

    private static long   CONFIG_KEY     = 0xb565a75eda53298eL ;

    //-------------------------------------------------------------- Private data

    private   Thread                 thread                   = null  ;
    private   AppScreen              appScreen                = null  ;
    private   RichTextField          statusField              = null  ;
    private   BlackBerryPIMList      listContact              = null  ;
    private   BlackBerryPIMList      listEvent                = null  ;
    private   Sync4jContactListener  listenerContact          = null  ;
    private   Sync4jEventListener    listenerEvent            = null  ;

    private   Hashtable              properties               = null  ;

    private   StaticDataHelper       sdh                      = null  ;

    private   String                 contactSourceType        = null  ;
    private   String                 contactSourceName        = null  ;

    private   String                 calendarSourceType       = null  ;
    private   String                 calendarSourceName       = null  ;

    private   static  boolean        inSynchronize            = false ;

    static {
        store = PersistentStore.getPersistentObject(CONFIG_KEY);
    }

    /**
     * This is a default constructor for SyncClient class.
     * It internally opens the blackberry contact list and add's a listener
     * to the blackberry contact list and pushes the Home screen for SyncContacts.
     */
    public SyncClient()
    throws Exception {

        //
        // load xml language file
        //
        sdh = new StaticDataHelper();
        sdh.loadLanguage(this.getClass());

        //
        //Load default properties form jad file
        //
        sdh.loadDefaultValue();

        listContact = (BlackBerryPIMList)
                            PIM.getInstance().
                                openPIMList(PIM.CONTACT_LIST, PIM.READ_WRITE);

        listEvent   = (BlackBerryPIMList)
                            PIM.getInstance().
                                openPIMList(PIM.EVENT_LIST  , PIM.READ_WRITE);

        listenerContact = new Sync4jContactListener ();
        listenerEvent   = new Sync4jEventListener   ();

        statusField = new RichTextField("",Field.NON_FOCUSABLE) ;
        appScreen   = new AppScreen()                           ;

        listContact.addListener(listenerContact) ;
        //listEvent.addListener(listenerEvent)     ;

        synchronized(store) {

            String[] contents = (String[])store.getContents();

            if(contents == null) {

                displayStatus(sdh.getLanguage("conf-not-set"));

            } else {

                String syncContact  = contents[3]   ;
                String syncCalendar = contents[4]   ;

                if (!("true".equals(syncContact)) && !("true".equals(syncCalendar))) {
                    displayStatus(sdh.getLanguage("nothing-sync-enable"));
                }
            }

        }

        synchronized(getEventLock()) {
            pushScreen(appScreen);
        }
    }

    /**
     * This method is invoked when the synchronize button on Home Screen is
     * clicked.
     *
     * @param void
     * @return void
     */
    public void synchronize()
    throws Exception {
        try {



            String[] contents = null;

            contactSourceType  = CONTACT_SORCE_TYPE   ;
            contactSourceName  = CONTACT_SOURCE_NAME  ;
            calendarSourceType = CALENDAR_SOURCE_TYPE ;
            calendarSourceName = CALENDAR_SOURCE_NAME ;

            synchronized(store) {
                contents = (String[])store.getContents();
                if(contents == null) {
                    displayStatus(sdh.getLanguage("conf-not-set"));
                    return;
                }
            }

            final String serverURL         = contents[ 0  ]  ;
            final String contactSourceURI  = contents[ 1  ]  ;
            final String calendarSourceURI = contents[ 2  ]  ;
            final String syncContact       = contents[ 3  ]  ;
            final String syncCalendar      = contents[ 4  ]  ;
            final String gatewayApn        = contents[ 7  ]  ;
            final String gatewayIp         = contents[ 8  ]  ;
            final String gatewayPort       = contents[ 9  ]  ;

            final String login             = contents[ 5  ]  +
                                             ":"            +
                                             contents[ 6  ]  ;

            if (!("true".equals(syncContact)) && !("true".equals(syncCalendar))) {
                displayStatus(sdh.getLanguage("nothing-sync-enable")              +
                                              ":\n"                               +
                                              sdh.getLanguage("please-configure") );
                return;
            }

            sdh.setCalendarSourceUri (calendarSourceURI ) ;
            sdh.setContactSourceUri  (contactSourceURI  ) ;

            sdh.setOS();

            properties = new Hashtable();

            properties.put(PROP_SERVER_URL  ,  serverURL    );
            properties.put(PROP_LOGIN       ,  login        );
            properties.put(PROP_GATEWAYAPN  ,  gatewayApn   );
            properties.put(PROP_GATEWAYIP   ,  gatewayIp    );
            properties.put(PROP_GATEWAYPORT ,  gatewayPort  );

            boolean signal = true;

            if ((RadioInfo.getState() != RadioInfo.STATE_ON) ||
                (RadioInfo.getSignalLevel() == RadioInfo.LEVEL_NO_COVERAGE)) {
                signal = false;
            }

            if (!signal) {
                if (RadioInfo.getState() == RadioInfo.STATE_OFF) {
                    displayStatus(sdh.getLanguage("no_network_available"));
                } else {
                    displayStatus(sdh.getLanguage("no_network_available_later"));
                }
            }

            if (signal && ("true".equals(syncContact) || "true".equals(syncCalendar))) {

                Thread syncThread = new Thread(){
                    public synchronized void run() {

                        if (inSynchronize) {
                            return;
                        }

                        inSynchronize = true;

                        if ("true".equals(syncContact)) {
                            displayStatus(sdh.getLanguage("synch-contact-wait") +
                                          "\n"                                  +
                                          sdh.getLanguage("please-wait")        );
                            listContact.removeListener(listenerContact);
                            try {

                                SyncManagerFactory.getSyncManager(contactSourceURI,
                                                                  contactSourceType,
                                                                  contactSourceName,
                                                                  properties).sync();
                                listContact.addListener(listenerContact);

                            } catch(Exception e) {
                                displayStatus(sdh.getLanguage("synch-failed")   +
                                              " "                               +
                                              getSynchError(e.getMessage()  ,
                                                            "address book"  ,
                                                            contactSourceURI)   );
                                return;
                            }
                        }

                        /**
                        if ("true".equals(syncCalendar)) {
                            displayStatus(sdh.getLanguage("synch-calendar-wait") +
                                          "\n"                                   +
                                          sdh.getLanguage("please-wait")         );
                            listEvent.removeListener(listenerEvent);

                            try {

                                SyncManagerFactory.getSyncManager(calendarSourceURI,
                                                                  calendarSourceType,
                                                                  calendarSourceName,
                                                                  properties).sync();
                                listEvent.addListener(listenerEvent);

                            } catch(Exception e) {
                                displayStatus(sdh.getLanguage("synch-failed")   +
                                              " "                               +
                                              getSynchError(e.getMessage()  ,
                                                            "calendar"      ,
                                                            calendarSourceURI)  );
                                return;
                            }
                        }

                        */

                        displayStatus(sdh.getLanguage("synch-success"));

                        inSynchronize = false;

                    }
                };

                syncThread.start();

            }




        } catch(Exception e) {
            displayStatus(sdh.getLanguage("synch-failed") + " " + e.getMessage());
            throw e;
        }
    }

    /**
     * This method is invoked to set/change the status message on the home screen.
     *
     * @param String: Message
     * @return void
     */
    void displayStatus(String message) {
      synchronized(getEventLock()) {

          AppScreen as = null;

          statusField = new RichTextField("", Field.NON_FOCUSABLE) ;
          as          = new AppScreen()                            ;
          statusField.setText(message);
          as.setStatus(statusField);
          pushScreen(as);

      }
    }

    //--captures Trackwheel click events, provided by TrackwheelListener interface
    public boolean trackwheelClick(int status, int line) {
        return true;
    }

    //--captures Trackwheel unclick events, provided by TrackwheelListener interface
    public boolean trackwheelUnclick(int status, int line) {
        return true;
    }

    //--captures Trackwheel roll events, provided by TrackwheelListener interface
    public boolean trackwheelRoll(int amount,int status, int line) {
        return true;
    }

    /**
     * Over ridden run method of Runnable interface.
     * A seperate thread is created to synchronize data with sync server
     *
     * @param void
     * @return void
     */
    public void run() {
        try {

            Thread mythread = Thread.currentThread();

            if (thread == mythread) {

                synchronized (this) {


                   synchronize();

                }
            }

        } catch(Exception ex) {
           displayStatus(sdh.getLanguage("exch-while-synch") + " " + ex);
        }
    }

    /**
     * return error message
     * @param error
     * @param remoteDBType
     * @param sourceUri
     */
    private String getSynchError(String error        ,
                                 String remoteDBType ,
                                 String sourceUri    ){

        if (ERROR_NOT_AUTHORIZED.equals(error)) {

            return sdh.getLanguage("not-authorized-user-and-password");

        } else if (ERROR_NOT_FOUND.equals(error)) {

            return sdh.getLanguage("not-found-remote-syncsource") +
                                   ": "                           +
                                   sourceUri                      ;

        }  else if (error != null && error.indexOf("Connection refused") != -1) {

            return sdh.getLanguage("error-connecting-to-the-server");

        } else if (error != null && error.indexOf("out-of-memory") != -1) {

            return sdh.getLanguage("out-of-memory");

        } else {

            return error;

        }

    }

    /**
     * <code>AppScreen</code> is an inner class that creates the home screen.
     * for syncClient on Blackberry.Extends MainScreen.
     */
    class AppScreen extends MainScreen {
        //-------------------Private UI components
        private ConfigurationScreen configScreen;

        StaticDataHelper sdh = new StaticDataHelper() ;

        //------Constructor
        /**
        *  The constructor here creates the home screen with all the UI components on it.
        */
        public AppScreen() {
            Bitmap icon            = null ;
            BitmapField iconField  = null ;

            icon      = Bitmap.getBitmapResource(IMMAGE_LOGO);
            iconField = new BitmapField(icon);

            iconField.setSpace(76, 18);

            configScreen = new ConfigurationScreen();

            setTitle (new LabelField(sdh.getLanguage("sync4j-bb-syncclient")   ) );

            add      (new LabelField(sdh.getLanguage("goto-menu-for-changing") ) );
            add      (new LabelField(sdh.getLanguage("or-to-synchronize")      ) );
            add      (iconField);

        }

        /**
         * This creates a MenuItem containing "Configure" option.
         * on selecting the configure option. The user is directed to configuration screen.
         */
        private MenuItem configMenu = new MenuItem(sdh.getLanguage("configure"), 20000, 10) {

            public void run() {
                synchronized(getEventLock()) {
                    pushScreen(configScreen);
                }
            }

        };

        /**
         * This creates a MenuItem containing "Synchronize.." option.
         * on selecting the configure option.
         * The users information in address book is synchronized.
         */
        private MenuItem SyncMenu = new MenuItem(sdh.getLanguage("synchronize"), 10000, 9) {
            public void run() {
                synchronized(this) {
                    thread = new Thread(SyncClient.this);
                    thread.start();
                }
            }
        };

        /**
         * Adds "Configure"  & "synchronize" menu items to the menu on Home Screen
         * @param Menu : menu to which configure & synchronize menuitems are
         * to be added
         * @param int : instance
         * @return void.
         */
        public void makeMenu(Menu menu, int instance) {
            menu.add(SyncMenu   );
            menu.add(configMenu );
        }

        /**
         * This method is invoked on user hitting the escape button when in home screen.
         * It prompts the user with a YES/NO dialog, if he wants to exit/not.
         * @param void.
         * @return void.
         */
        public boolean onClose()
        {
            System.exit(0);
            return true;
        }
    }

    /**
     * <code>ConfigurationScreen</code> is an inner class that creates configuration screen.
     * for configuring parameters required for sync Client.Extends MainScreen.
     */
    class ConfigurationScreen extends MainScreen {

        StaticDataHelper sdh = new StaticDataHelper() ;

        //-------------------Private UI components
        private BasicEditField     homePageField          = null ;
        private BasicEditField     userNameField          = null ;
        private BasicEditField     contactSourceUriField  = null ;
        private BasicEditField     calendarSourceUriField = null ;
        private PasswordEditField  passwordField          = null ;
        private BasicEditField     gatewayApnField        = null ;
        private BasicEditField     gatewayIpField         = null ;
        private BasicEditField     gatewayPortField       = null ;
        private CheckboxField      syncContactField       = null ;
        private CheckboxField      syncCalendarField      = null ;

        //------Constructor
        /**
         *  The constructor here creates the Configuration screen with all the UI components on it.
         *  and setting default input values.
         */
        public ConfigurationScreen() {


            setTitle(new LabelField(sdh.getLanguage("configure-sync4j-bb")));


            userNameField           = new BasicEditField
                                        (sdh.getLanguage("user") + " "                ,
                                        sdh.getUserDefault()                          );

            passwordField           = new PasswordEditField
                                        (sdh.getLanguage("password") + " "            ,
                                        sdh.getPasswordDefault()                      );

            homePageField           = new BasicEditField
                                        (sdh.getLanguage("homepage") + " \n"          ,
                                        sdh.getHomePageDefault()                      );

            contactSourceUriField   = new BasicEditField
                                        (sdh.getLanguage("source-uri-contact") + " "  ,
                                        sdh.getContactSourceUriDefault()              );

            calendarSourceUriField  = new BasicEditField
                                        (sdh.getLanguage("source-uri-calendar") + " " ,
                                        sdh.getCalendarSourceUriDefault()             );

            syncContactField        = new CheckboxField
                                        (sdh.getLanguage("enable-sync-contact")       ,
                                        sdh.getSyncContactDefault()                   );

            syncCalendarField       = new CheckboxField
                                        (sdh.getLanguage("enable-sync-calendar")      ,
                                        sdh.getSyncCalendarDefault()                  );

            gatewayApnField         = new BasicEditField
                                        (sdh.getLanguage("gateway-apn") + " "        ,
                                        sdh.getGatewayApnDefault()                   );

            gatewayIpField          = new BasicEditField
                                        (sdh.getLanguage("gateway-ip") + " "         ,
                                        sdh.getGatewayIpDefault()                    );

            gatewayPortField        = new BasicEditField
                                        (sdh.getLanguage("gateway-port") + " "      ,
                                        sdh.getGatewayPortDefault()                 ,
                                        6                                           ,
                                        EditField.FILTER_INTEGER                    );

            SeparatorField separator1 = new SeparatorField();
            SeparatorField separator2 = new SeparatorField();
            SeparatorField separator3 = new SeparatorField();
            SeparatorField separator4 = new SeparatorField();

            add(userNameField            );
            add(passwordField            );
            add(separator1               );
            add(homePageField            );
            //add(calendarSourceUriField   );
            add(separator2               );
            add(contactSourceUriField    );
            add(syncContactField         );
            //add(syncCalendarField        );
            //add(separator3               );
            add(separator4               );
            add(gatewayApnField          );
            add(gatewayIpField           );
            add(gatewayPortField         );

            populateData();
        }

        /**
         * populates the UI components with previous settings, if available.
         * @param void.
         * @return void.
         */
        private void populateData() {
            String[] data = null;

            synchronized(store) {
                data = (String[])store.getContents();
            }

            if(data != null) {

               homePageField.setText          (data[0]);
               contactSourceUriField.setText  (data[1]);
               calendarSourceUriField.setText (data[2]);

               if ("true".equals(data[3])) {
                   syncContactField.setChecked  (true  );
               } else {
                   syncContactField.setChecked  (false );
               }

               if ("true".equals(data[4])) {
                   //syncCalendarField.setChecked (true  );
                   syncCalendarField.setChecked (false  );
               } else {
                   syncCalendarField.setChecked (false );
               }

               userNameField.setText          (data[5]  ) ;
               passwordField.setText          (data[6]  ) ;
               gatewayApnField.setText        (data[7]  ) ;
               gatewayIpField.setText         (data[8]  ) ;
               gatewayPortField.setText       (data[9]  ) ;

            }
        }

        /**
         * This creates a MenuItem containing "Save" option.
         * on selecting the save option the configuration details are saved.
         */
        private MenuItem saveConfItem = new MenuItem(sdh.getLanguage("save"), 20000, 10) {

            public void run() {
                saveConfiguration();
            }

        };

        /**
         * This method is called when a user selects save option from the menu in configuration screen.
         * or from the dialog when the user exits configuration screen.It saves changes in configuration
         * into persistant store.
         * @param void.
         * @return void.
         */
        public void saveConfiguration(){

            String homePage           = null ;
            String userName           = null ;
            String password           = null ;
            String gatewayApn         = null ;
            String gatewayIp          = null ;
            String gatewayPort        = null ;
            String contactSourceUri   = null ;
            String calendarSourceUri  = null ;
            String syncContact        = null ;
            String syncCalendar       = null ;

            homePage           = homePageField.getText           ().trim();
            contactSourceUri   = contactSourceUriField.getText   ().trim();
            calendarSourceUri  = calendarSourceUriField.getText  ().trim();

            if (syncContactField.getChecked())  {
                syncContact = "true";
            } else {
                syncContact = "false";
            }

            if (syncCalendarField.getChecked()) {
                //syncCalendar = "true";
                syncCalendar = "false";
            } else {
                syncCalendar = "false";
            }

            userName           = userNameField.getText           ().trim() ;
            password           = passwordField.getText           ().trim() ;
            gatewayApn         = gatewayApnField.getText         ().trim() ;
            gatewayIp          = gatewayIpField.getText          ().trim() ;
            gatewayPort        = gatewayPortField.getText        ().trim() ;

            if (homePage          == null ||
                contactSourceUri  == null ||
                calendarSourceUri == null ||
                userName          == null ||
                password          == null) {

                   displayStatus(sdh.getLanguage("configuration-incomplete"));

            } else if (homePage.equals   ("") ||
                contactSourceUri.equals  ("") ||
                calendarSourceUri.equals ("") ||
                userName.equals          ("") ||
                password.equals          ("")) {

                displayStatus(sdh.getLanguage("configuration-incomplete"));

            } else if ((homePage.indexOf("http") != 0) && (homePage.indexOf("https") != 0)) {

                displayStatus(sdh.getLanguage("homepage-not-correct"));

            } else {

                    String[] data = {homePage          ,
                                     contactSourceUri  ,
                                     calendarSourceUri ,
                                     syncContact       ,
                                     syncCalendar      ,
                                     userName          ,
                                     password          ,
                                     gatewayApn        ,
                                     gatewayIp         ,
                                     gatewayPort };

                    synchronized(store) {
                        store.setContents(data);
                        store.commit();
                    }

            }

            popScreen(ConfigurationScreen.this);
        }

        /**
         * Adds "Save" menu item to the menu on Configuration screen
         * @param Menu : menu to which save menuitem is to be added
         * @param int : instance
         * @return void.
         */
        public void makeMenu(Menu menu, int instance) {
            menu.add(saveConfItem);
        }

        /**
         * This method is invoked on user hitting the escape button.
         * It prompts the user if with a dialog,if he wants to save/discard/cancel
         * the changes in configuration.
         * @param void.
         * @return void.
         */
        public boolean onClose() {

            int response = Dialog.ask(Dialog.D_SAVE,sdh.getLanguage("do-you-want-save"));

            if (Dialog.SAVE == response ) {
                    saveConfiguration();
            } else if( Dialog.CANCEL == response) {
                    displayStatus(sdh.getLanguage("no-changes-to-previous"));
            } else if ( Dialog.DISCARD == response ) {
                    popScreen(ConfigurationScreen.this);
                    displayStatus(sdh.getLanguage("no-changes-to-previous"));
            }

            return true;
        }
    }

    /**
     * This is the entry point for SyncClient Application.
     * @param String[].
     * @return void.
     */
    public static void main(String s[]) throws Exception {
        SyncClient test = new SyncClient();
        test.enterEventDispatcher();
    }

}
