/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.syncclient.blackberry.listener;

import javax.microedition.pim.Contact;
import javax.microedition.pim.Event;
import javax.microedition.pim.PIMItem;
import net.rim.blackberry.api.pdap.PIMListListener;

import sync4j.syncclient.sps.EventDataStore;

/**
 * <code>Sync4jPIMListener</code> listens for changes to the BlackBerry
 * contact database and records the changes to a persistent store. The stored
 * information is used for syncing with sync4j subsequently
 *
 * @author Fabio Maggi @ Funambol
 * $Id
 */
public class Sync4jEventListener implements PIMListListener{

    public static final char   RECORD_STATE_DELETED  = 'D' ;

    //Data store object
    private EventDataStore eventDataStore ;

    /**
     * Default Constructor for Sync4jPIMListener class
     * that initializes the store object.
     *
     * @param void
     *
     */
    public Sync4jEventListener() {
        this.eventDataStore = new EventDataStore(); 
    }

    /**
     * This method is invoked when the item is added to blackberry addressbook.
     * @param PIMItem ,item that was added to the address book
     * @return void
     */
    public void itemAdded(PIMItem item) {
        //
        // this code is disable because
        // blackberry bug in calendar event
        //
        //storeRecord(item, EventDataStore.RECORD_STATE_NEW);
    }

    /**
     * This method is invoked when the item is updated in blackberry addressbook.
     * @param PIMItem oldItem, item that was changed from the address book
     * @param PIMItem newItem, item that is changed from the address book
     * @return void
     */
    public void itemUpdated(PIMItem oldItem, PIMItem newItem) {
        //
        // this code is disable because
        // blackberry bug in calendar event
        //
        //storeRecord(newItem, EventDataStore.RECORD_STATE_UPDATED);
    }

    /**
     * This method is invoked when an item is removed from blackberry calendar.
     * @param PIMItem item that was removed from the calendar
     * @return void
     */
    public void itemRemoved(PIMItem item) {
        //
        // this code is disable because
        // blackberry bug in calendar event
        //
        //storeRecord(item, EventDataStore.RECORD_STATE_DELETED);
    }

    /**
     * This method is invoked internally when any changes to the
     * blackberry address book happen.It updates the information into
     * persistant store.
     *
     * @param PIMItem item that was removed from the address book
     * @return void
     */
    private void storeRecord(PIMItem item, char state) {

        Event  event = (Event) item ;
        String uid   = null         ;

        try {

            uid  = event.getString(Event.UID, 0);

            eventDataStore.addRecord(uid, state, event);

        } catch(Exception e){
            e.printStackTrace();
        }

    }
}
