/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.syncclient.blackberry.parser;

import javax.microedition.pim.Contact;
import javax.microedition.pim.ContactList;
import javax.microedition.pim.Event;
import javax.microedition.pim.EventList;

/**
 * <code>ParserFactory</code> used for obtaining an instance of ContactParser.
 *
 * @author Fabio Maggi @ Funambol
 * $Id: ParserFactory.java,v 1.0
 */

public class ParserFactory
{
   /**
    * This method is invoked to obtain an instance of XMLContactParser.
    * the XMLContactParser implements ContactParser interface.
    *
    * @param ContactList: Blackberry contact list
    * @param Contact: Blackberry contact
    * @param boolean: to modify or not to modify
    * @return XMLContactParser
    */
   public static final ContactParser getParserInstance(ContactList list, Contact contact, boolean shouldModify) {
       return new XMLContactParser(list, contact, shouldModify);
   }
   
   /**
    * This method is invoked to obtain an instance of XMLEventParser.
    * the XMLEventParser implements EventParser interface.
    *
    * @param EventList: Blackberry event list
    * @param Event: Blackberry event
    * @param boolean: to modify or not to modify
    * @return XMLEventParser
    */
   public static final EventParser getParserInstance(EventList list, Event event, boolean shouldModify) {
       return new XMLEventParser(list, event, shouldModify);
   }
   
}
