/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.syncclient.blackberry.parser;

import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.pim.Contact;
import javax.microedition.pim.ContactList;
import javax.microedition.pim.PIMItem;

import sync4j.syncclient.common.StringTools;

import sync4j.syncclient.util.StaticDataHelper;

import net.rim.device.api.ui.component.Dialog;

/**
 * <code>XMLContactParser</code> provides with the user interface
 * for sync contacts with SyncServer.
 *
 * @author santoshs email:info@telibrahma.com
 * @author Fabio Maggi @ Funambol
 * $Id: XMLContactParser.java,v 1.0
 */

public class XMLContactParser implements ContactParser{

        //------------------------------------------Private Data Constants

    private static final String FIRST_NAME        = "FirstName"                 ;
    private static final String LAST_NAME         = "LastName"                  ;
    private static final String TITLE             = "JobTitle"                  ;
    private static final String COMPANY           = "CompanyName"               ;
    private static final String EMAIL_1           = "Email1Address"             ;
    private static final String INSTANT_MESSENGER = "IstantMessenger"           ;
    private static final String WEB_PAGE          = "WebPage"                   ;
    private static final String MOBILE_TEL        = "MobileTelephoneNumber"     ;
    private static final String HOME_FAX          = "HomeFaxNumber"             ;
    private static final String BUSINESS_FAX      = "BusinessFaxNumber"         ;
    private static final String BUSINESS_TEL      = "BusinessTelephoneNumber"   ;
    private static final String BUSINESS_TEL2     = "Business2TelephoneNumber"  ;
    private static final String HOME_TEL          = "HomeTelephoneNumber"       ;
    private static final String BUSINESS_STREET   = "BusinessAddressStreet"     ;
    private static final String BUSINESS_CITY     = "BusinessAddressCity"       ;
    private static final String BUSINESS_STATE    = "BusinessAddressState"      ;
    private static final String BUSINESS_COUNTRY  = "BusinessAddressCountry"    ;
    private static final String BUSINESS_PO       = "BusinessAddressPostalCode" ;
    private static final String HOME_STREET       = "HomeAddressStreet"         ;
    private static final String HOME_CITY         = "HomeAddressCity"           ;
    private static final String HOME_STATE        = "HomeAddressState"          ;
    private static final String HOME_COUNTRY      = "HomeAddressCountry"        ;
    private static final String HOME_PO           = "HomeAddressPostalCode"     ;
    private static final String NOTE              = "Body"                      ;

    private static final String START_MARKER     = "<contact>"                  ;
    private static final String END_MARKER       = "</contact>"                 ;

    private static final String HIGHER_OS_4_0    = "4.0.2"                      ;
    private static final String HIGHER_OS_4_1    = "4.1"                        ;

    //------------------------------------------Private Data menmbers

    private String[] tagArray = {
            FIRST_NAME         ,
            LAST_NAME          ,
            EMAIL_1            ,
            COMPANY            ,
            TITLE              ,
            INSTANT_MESSENGER  ,
            WEB_PAGE           ,
            MOBILE_TEL         ,
            HOME_FAX           ,
            BUSINESS_FAX       ,
            BUSINESS_TEL       ,
            BUSINESS_TEL2      ,
            HOME_TEL           ,
            BUSINESS_STREET    ,
            BUSINESS_CITY      ,
            BUSINESS_STATE     ,
            BUSINESS_COUNTRY   ,
            BUSINESS_PO        ,
            HOME_STREET        ,
            HOME_CITY          ,
            HOME_STATE         ,
            HOME_COUNTRY       ,
            HOME_PO            ,
            NOTE
    };

    private String[] telAttrs = {
            MOBILE_TEL   ,
            BUSINESS_FAX ,
            BUSINESS_TEL ,
            HOME_TEL
    };

    private ContactList list;
    private Contact contact;
    private boolean modify;

    /**
     * This is an initialization constructor for XMLContactParser class.
     *
     * @param ContactList: Blackberry contact list
     * @param Contact: Blackberry contact
     * @param boolean: to modify or not to modify
     * @return XMLContactParser
     */
    public XMLContactParser(ContactList list, Contact contact, boolean modify) {
        this.list = list;
        this.contact = contact;
        this.modify = modify;
    }

    /**
     * This method is invoked to obtain Contact object from a contact String.
     * if the modify flag is set to true it modifies the contact.
     * if the modify flag is set to false it simply adds a contact to blackberry addressbook.
     *
     * @param String:String containing contact information to be parsed.
     * @return Contact: that has been added newly/modified in blackberry addressbook.
     */
    public Contact parseContact(String contactString) {
        if(modify)
            return modifyContact(contactString);
        else
            return addContact(contactString);
    }

    /**
     * This method is invoked to obtain Contact object from a contact String,
     * when a contact is added to the blackberry addressbook.This method
     * adds a new contact and returns the reference of that object.
     *
     * @param String: String containing contact information to be parsed.
     * @return Contact: Contact object that has been added newly.
     */
    public Contact addContact(String contactString) {

        Hashtable contactMap = buildMapFromXML(contactString);

        if (contactMap != null) {
            try {

                StaticDataHelper sdh       = null ;
                String           osVersion = null ;

                sdh = new StaticDataHelper();

                osVersion = sdh.getOS();

                String[] arrayField = new String[5];

                String ln = getValue(contactMap, LAST_NAME);
                String fn = getValue(contactMap, FIRST_NAME);
                
                arrayField[Contact.NAME_FAMILY] = ln;
                arrayField[Contact.NAME_GIVEN]  = fn;

                if (list.isSupportedField(Contact.NAME)) {
                     contact.addStringArray(Contact.NAME, PIMItem.ATTR_NONE, arrayField);
                }

                String field = getValue(contactMap, EMAIL_1);

                if (isSupportedField(Contact.EMAIL, field)) {
                    contact.addString(Contact.EMAIL, Contact.ATTR_NONE, field);
                }

                field = getValue(contactMap, NOTE);
                if (isSupportedField(Contact.NOTE, field)) {
                    contact.addString(Contact.NOTE, Contact.ATTR_NONE, field);
                }

                field = getValue(contactMap, TITLE);
                if (isSupportedField(Contact.TITLE, field)) {
                    contact.addString(Contact.TITLE, Contact.ATTR_NONE, field);
                }

                field = getValue(contactMap, COMPANY);
                //
                // void contact must be have firstName or LastName or company
                //                
                if (ln.length() == 0 && fn.length() == 0 && field.length() == 0) {
                    field = " ";
                }
                
                if (isSupportedField(Contact.ORG, field)) {
                    contact.addString(Contact.ORG, Contact.ATTR_NONE, field);
                }

                field = getValue(contactMap, WEB_PAGE);
                if (isSupportedField(Contact.URL, field)) {
                    contact.addString(Contact.URL, Contact.ATTR_NONE, field);
                }

                field = getValue(contactMap, HOME_TEL);
                if (isSupportedAtrributedField(Contact.TEL, Contact.ATTR_HOME, field)) {
                        contact.addString(Contact.TEL, Contact.ATTR_HOME, field);
                }

                field = getValue(contactMap, MOBILE_TEL);
                if (isSupportedAtrributedField(Contact.TEL, Contact.ATTR_MOBILE, field)) {
                        contact.addString(Contact.TEL, Contact.ATTR_MOBILE, field);
                }

                field = getValue(contactMap, BUSINESS_TEL);
                if (isSupportedAtrributedField(Contact.TEL, Contact.ATTR_WORK, field)) {
                        contact.addString(Contact.TEL, Contact.ATTR_WORK, field);
                }

                field = getValue(contactMap, BUSINESS_FAX);

                if (isSupportedAtrributedField(Contact.TEL, Contact.ATTR_FAX, field)) {
                        contact.addString(Contact.TEL, Contact.ATTR_FAX, field);
                }

                if (checkOS(osVersion)) {

                    arrayField = new String[7];
                    arrayField[Contact.ADDR_COUNTRY]     = getValue(contactMap, BUSINESS_COUNTRY );
                    arrayField[Contact.ADDR_STREET]      = getValue(contactMap, BUSINESS_STREET  );
                    arrayField[Contact.ADDR_REGION]      = getValue(contactMap, BUSINESS_STATE   );
                    arrayField[Contact.ADDR_LOCALITY]    = getValue(contactMap, BUSINESS_CITY    );
                    arrayField[Contact.ADDR_POSTALCODE]  = getValue(contactMap, BUSINESS_PO      );

                    if (list.isSupportedField(Contact.ADDR)) {
                        contact.addStringArray(Contact.ADDR, Contact.ATTR_WORK, arrayField);
                    }

                    arrayField = new String[7];
                    arrayField[Contact.ADDR_COUNTRY]    = getValue(contactMap, HOME_COUNTRY );
                    arrayField[Contact.ADDR_STREET]     = getValue(contactMap, HOME_STREET  );
                    arrayField[Contact.ADDR_REGION]     = getValue(contactMap, HOME_STATE   );
                    arrayField[Contact.ADDR_LOCALITY]   = getValue(contactMap, HOME_CITY    );
                    arrayField[Contact.ADDR_POSTALCODE] = getValue(contactMap, HOME_PO      );
                    if (list.isSupportedField(Contact.ADDR) && list.isSupportedAttribute(Contact.ADDR, Contact.ATTR_HOME))
                            contact.addStringArray(Contact.ADDR, Contact.ATTR_HOME, arrayField);

                }

                return contact;

            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }
        return null;
    }

    /**
     * This method is invoked to obtain Contact object from a contact String,
     * when a contact is modified in the blackberry addressbook.This method
     * modifies the object and returns the reference to that object.
     *
     * @param String: String containing contact information to be parsed.
     * @return Contact:Contact that has been modified.
     */
    public Contact modifyContact(String contactString) {

        Hashtable contactMap = buildMapFromXML(contactString);

        StaticDataHelper sdh       = null ;
        String           osVersion = null ;

        sdh = new StaticDataHelper();

        osVersion = sdh.getOS();

        if (contactMap != null) {
            try {
                String[] arrayField = new String[5];
                
                String fn = getValue(contactMap, FIRST_NAME );
                String ln = getValue(contactMap, LAST_NAME  );
                
                arrayField[Contact.NAME_FAMILY] = ln;
                arrayField[Contact.NAME_GIVEN]  = fn;
                
                if(contact.countValues(Contact.NAME) > 0) {
                    contact.setStringArray(Contact.NAME, 0, Contact.ATTR_NONE, arrayField);
                } else {
                    contact.addStringArray(Contact.NAME, Contact.ATTR_NONE, arrayField);
                }

                String field = getValue(contactMap, EMAIL_1);
                if (isSupportedField(Contact.EMAIL, field)) {
                    if (contact.countValues(Contact.EMAIL) > 0) {
                        contact.setString(Contact.EMAIL, 0, Contact.ATTR_NONE, field);
                    } else {
                        contact.addString(Contact.EMAIL, Contact.ATTR_NONE, field);
                    }
                }
                
                field = getValue(contactMap, NOTE);
                if (isSupportedField(Contact.NOTE, field)) {
                    if (contact.countValues(Contact.NOTE) > 0) {
                        contact.removeValue(Contact.NOTE, 0);
                        contact.addString(Contact.NOTE, Contact.ATTR_NONE, field);
                    } else {
                        contact.addString(Contact.NOTE, Contact.ATTR_NONE, field);
                    }
                }

                field = getValue(contactMap, COMPANY);
                //
                // void contact must be have firstName or LastName or company
                //
                if (ln.length() == 0 && fn.length() == 0 && field.length() == 0) {
                    field = " ";
                }
              
                if (isSupportedField(Contact.ORG, field)) {
                    if (contact.countValues(Contact.ORG) > 0) {
                        contact.setString(Contact.ORG, 0, Contact.ATTR_NONE, field);
                    } else {
                        contact.addString(Contact.ORG, Contact.ATTR_NONE, field);
                    }
                }
                
                field = getValue(contactMap, TITLE);
                if (isSupportedField(Contact.TITLE, field) && contact.countValues(Contact.TITLE) > 0) {
                    contact.setString(Contact.TITLE, 0, Contact.ATTR_NONE, field);
                } else {
                    contact.addString(Contact.TITLE, Contact.ATTR_NONE, field);
                }

                if( isSupportedField(Contact.URL, field) && contact.countValues(Contact.URL) > 0) {
                    contact.removeValue(Contact.URL, 0);
                    field = getValue(contactMap, WEB_PAGE);
                    contact.addString(Contact.URL, Contact.ATTR_NONE, field);
                }

                int telValues = contact.countValues(Contact.TEL);
                for(int i=0; i<telValues;i++) {
                    contact.removeValue(Contact.TEL, 0);
                }

                field = getValue(contactMap, HOME_TEL);
                if (isSupportedAtrributedField(Contact.TEL, Contact.ATTR_HOME, field))
                        contact.addString(Contact.TEL, Contact.ATTR_HOME, field);

                field = getValue(contactMap, MOBILE_TEL);
                if (isSupportedAtrributedField(Contact.TEL, Contact.ATTR_MOBILE, field))
                        contact.addString(Contact.TEL, Contact.ATTR_MOBILE, field);

                field = getValue(contactMap, BUSINESS_TEL);
                if (isSupportedAtrributedField(Contact.TEL, Contact.ATTR_WORK, field))
                        contact.addString(Contact.TEL, Contact.ATTR_WORK, field);

                field = getValue(contactMap, BUSINESS_FAX);
                if (isSupportedAtrributedField(Contact.TEL, Contact.ATTR_FAX, field))
                        contact.addString(Contact.TEL, Contact.ATTR_FAX, field);

                if (checkOS(osVersion)) {

                    arrayField = new String[7];
                    arrayField[Contact.ADDR_COUNTRY]    = getValue(contactMap, BUSINESS_COUNTRY );
                    arrayField[Contact.ADDR_STREET]     = getValue(contactMap, BUSINESS_STREET  );
                    arrayField[Contact.ADDR_REGION]     = getValue(contactMap, BUSINESS_STATE   );
                    arrayField[Contact.ADDR_LOCALITY]   = getValue(contactMap, BUSINESS_CITY    );
                    arrayField[Contact.ADDR_POSTALCODE] = getValue(contactMap, BUSINESS_PO      );


                    boolean findWork = false;
                    int     i        = 0    ;

                    for (i=0; i < contact.countValues(Contact.ADDR); i++) {
                        String [] addr = contact.getStringArray(Contact.ADDR,i);
                        if ( addr != null && addr.length > 0 ) {
                            int addrType = contact.getAttributes(Contact.ADDR,i);

                            if (addrType == Contact.ATTR_WORK) {

                                //
                                // update
                                //
                                contact.setStringArray(Contact.ADDR, i, Contact.ATTR_WORK, arrayField);
                                findWork = true;

                            }

                        }

                    }

                    
                    //
                    // add
                    //
                    if (!findWork) {
                        contact.addStringArray(Contact.ADDR, Contact.ATTR_WORK, arrayField);
                    }

                    arrayField = new String[7];
                    arrayField[Contact.ADDR_COUNTRY]    = getValue(contactMap, HOME_COUNTRY );
                    arrayField[Contact.ADDR_STREET]     = getValue(contactMap, HOME_STREET  );
                    arrayField[Contact.ADDR_REGION]     = getValue(contactMap, HOME_STATE   );
                    arrayField[Contact.ADDR_LOCALITY]   = getValue(contactMap, HOME_CITY    );
                    arrayField[Contact.ADDR_POSTALCODE] = getValue(contactMap, HOME_PO      );


                    boolean findHome = false;

                    for (i=0; i < contact.countValues(Contact.ADDR); i++) {
                        String [] addr = contact.getStringArray(Contact.ADDR,i);
                        if ( addr != null && addr.length > 0 ) {
                            int addrType = contact.getAttributes(Contact.ADDR,i);

                            if (addrType == Contact.ATTR_HOME) {
                                //
                                // update
                                //
                                contact.setStringArray(Contact.ADDR, i, Contact.ATTR_HOME, arrayField);
                                findHome = true;

                            }

                        }

                    }

                    //
                    // add
                    //
                    if (!findHome) {
                        contact.addStringArray(Contact.ADDR, Contact.ATTR_HOME, arrayField);
                    }

                }

                return contact;
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }
        return null;
    }

    /**
     * This method appends all the contact information passed to it
     * in the form of contact object.
     *
     * @param Contact: Blackberry contact.
     * @return String: all contact information in a string format.
     */
    public String toString(Contact contact) {

        StringBuffer contactBuffer = new StringBuffer();

        StaticDataHelper sdh       = null ;
        String           osVersion = null ;

        sdh = new StaticDataHelper();

        osVersion = sdh.getOS();

        contactBuffer.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" );

        contactBuffer.append("<contact>\n");

        appendName            (contactBuffer     ,
                               contact           );

        appendField           (contactBuffer     ,
                               contact           ,
                               Contact.ORG       ,
                               COMPANY           );

        appendField           (contactBuffer     ,
                               contact           ,
                               Contact.NOTE      ,
                               NOTE              );

        appendField           (contactBuffer     ,
                               contact           ,
                               Contact.TITLE     ,
                               TITLE             );

        appendField           (contactBuffer     ,
                               contact           ,
                               Contact.EMAIL     ,
                               EMAIL_1           );

        appendField           (contactBuffer     ,
                               contact           ,
                               Contact.URL       ,
                               WEB_PAGE          );

        appendTelephoneFields (contactBuffer     ,
                               contact           );

        if (checkOS(osVersion)) {
            appendAddress      (contactBuffer     ,
                                contact           );
        }

        contactBuffer.append("</contact>");

        return contactBuffer.toString();
    }

    //-----------------------------------------Private Methods

    /**
     * This method appends the contact information supported by blackberry to a
     * string buffer.
     *
     * @param StringBuffer: StringBuffer containing contact information.
     * @param Contact: Blackberry contact
     * @param int: type of field
     * @param String: Name of field.
     * @return void.
     */
    private void appendField(StringBuffer contactBuffer, Contact contact, int field, String tag) {

        String value = null;

        if (list.isSupportedField(field)) {

            value = "";

            try {
                value = contact.getString(field, 0);
            } catch (Exception e) {
                StaticDataHelper.log("Not set field: " + tag);
            }
   
            //
            // void contact must be have firstName or LastName or company
            // if all fields missing company was setted to " "
            //            
            if (field == Contact.ORG && (value != null && value.equals(" "))) {
                value = "";
            }
            
            appendToContact(contactBuffer, tag, value);

        }
    }

    /**
     * This method appends the contact information supported by blackberry to a
     * string buffer.
     *
     * @param StringBuffer: StringBuffer containing contact information.
     * @param String: name of tag associated with contact information.
     * @param String: value corresponding to tag.
     * @return void
     */
    private void appendToContact(StringBuffer contactBuffer, String tag, String value) {

        value = (value == null) ? "" : value;

        if (value.length() > 0) {
            value = StringTools.escapeXml(value);
        }

        contactBuffer.append("<").append(tag).append(">").append(value);
        contactBuffer.append("</").append(tag).append(">\n");

    }

    /**
     * This method only appends the data corresponding to contact's name.
     *
     * @param StringBuffer: string buffer to which data is appended.
     * @param Contact: Blackberry contact
     * @return void
     */
    private void appendName(StringBuffer buffer, Contact contact) {
        if (contact.countValues(Contact.NAME) > 0) {
            String[] name = contact.getStringArray(Contact.NAME, 0);
            appendToContact(buffer, FIRST_NAME, name[Contact.NAME_GIVEN]);
            appendToContact(buffer, LAST_NAME, name[Contact.NAME_FAMILY]);
        } else {
            appendToContact(buffer, FIRST_NAME, "");
            appendToContact(buffer, LAST_NAME,  "");
        }
    }

    /**
     * This method only appends the data corresponding to contact's telephone numbers.
     *
     * @param StringBuffer: Blackberry contact list
     * @param Contact: Blackberry contact
     * @return void
     */
    private void appendTelephoneFields(StringBuffer buffer , Contact contact) {
        int totalTelFields = contact.countValues(Contact.TEL);
        Vector addedAttrs = new Vector();
        for(int i=0;i<totalTelFields;i++) {
            String value = contact.getString(Contact.TEL, i);
            int attr = contact.getAttributes(Contact.TEL, i);
            String tag = "";
            switch(attr) {
                case Contact.ATTR_MOBILE:
                    tag = MOBILE_TEL;
                    break;
                case Contact.ATTR_FAX:
                    tag = BUSINESS_FAX;
                    break;
                case Contact.ATTR_WORK:
                    tag = BUSINESS_TEL;
                    break;
                case Contact.ATTR_HOME:
                    tag = HOME_TEL;
                    break;
            }
            if(!tag.equals("")) {
                appendToContact(buffer, tag, value);
                addedAttrs.addElement(tag);
            }
        }

        for (int i = 0; i < telAttrs.length; i++) {
            if(!addedAttrs.contains(telAttrs[i]))
                appendToContact(buffer, telAttrs[i], "");
        }
    }

    /**
     * This method only appends the data corresponding to contact's address details.
     *
     * @param StringBuffer: Blackberry contact list
     * @param Contact: Blackberry contact
     * @return void
     */
    private void appendAddress(StringBuffer buffer, Contact contact) {

        StaticDataHelper sdh          = null  ;
        String           osVersion    = null  ;
        boolean          findHome     = false ;
        boolean          findBusiness = false ;

        sdh = new StaticDataHelper();

        osVersion = sdh.getOS();
        
        if (checkOS(osVersion)) {
        
            if (list.isSupportedField(Contact.ADDR) && contact.countValues(Contact.ADDR) > 0) {
    
                for (int i=0; i < contact.countValues(Contact.ADDR); i++) {
                    String [] addr = contact.getStringArray(Contact.ADDR,i);
                    if ( addr != null && addr.length > 0 ) {
                        int addrType = contact.getAttributes(Contact.ADDR,i);
    
                        if (addrType == Contact.ATTR_HOME) {
    
                            findHome = true;
                            
                            appendToContact(buffer, HOME_STREET  , addr[Contact.ADDR_STREET]     );
                            appendToContact(buffer, HOME_STATE   , addr[Contact.ADDR_REGION]     );
                            appendToContact(buffer, HOME_CITY    , addr[Contact.ADDR_LOCALITY]   );
                            appendToContact(buffer, HOME_PO      , addr[Contact.ADDR_POSTALCODE] );
                            appendToContact(buffer, HOME_COUNTRY , addr[Contact.ADDR_COUNTRY]    );
    
                        } else {
                            
                            findBusiness = true;
                            
                            appendToContact(buffer, BUSINESS_STREET  , addr[Contact.ADDR_STREET]     );
                            appendToContact(buffer, BUSINESS_STATE   , addr[Contact.ADDR_REGION]     );
                            appendToContact(buffer, BUSINESS_CITY    , addr[Contact.ADDR_LOCALITY]   );
                            appendToContact(buffer, BUSINESS_PO      , addr[Contact.ADDR_POSTALCODE] );
                            appendToContact(buffer, BUSINESS_COUNTRY , addr[Contact.ADDR_COUNTRY]    );
                        }
                    
                    }
                }   
            } 
    
            if (!findHome) {
                appendToContact(buffer, HOME_STREET  , ""   );
                appendToContact(buffer, HOME_STATE   , ""   );
                appendToContact(buffer, HOME_CITY    , ""   );
                appendToContact(buffer, HOME_PO      , ""   );
                appendToContact(buffer, HOME_COUNTRY , ""   );
            }
            
            if (!findBusiness) {
                appendToContact(buffer, BUSINESS_STREET  , ""  );
                appendToContact(buffer, BUSINESS_STATE   , ""  );
                appendToContact(buffer, BUSINESS_CITY    , ""  );
                appendToContact(buffer, BUSINESS_PO      , ""  );
                appendToContact(buffer, BUSINESS_COUNTRY , ""  );
            }
         
        }
    
    }

    /**
     * This method returns the value associated with a key in a hashtable
     * containing contactMap.
     *
     * @param Hashtable: contactMap
     * @param String: key to which associated value it to be found.
     * @return String: Value associated with the key
     */
    private String getValue(Hashtable contactMap, String key) {
        Object val = contactMap.get(key);
        String value = val == null ? "" : val.toString();
        return value.trim();
    }

    /**
     * This method checks if the field and attibute are supported by the contact list
     * of blackberry address book
     *
     * @param int: field
     * @param int: attribute
     * @param String: value
     * @return boolean:returns true if both the field and attribute
     * associated with it r supported
     */
    private boolean isSupportedAtrributedField(int field, int attribute, String value) {
        return isSupportedField(field, value) && list.isSupportedAttribute(field, attribute);
    }

    /**
     * This method checks if the type of field is supported by the contact list
     * of blackberry address book.
     *
     * @param int: type
     * @param String: value
     * @return boolean:returns true if contact list supports this field
     */
    private boolean isSupportedField(int type, String value) {
        return list.isSupportedField(type);
    }

    /**
     * This method creates a hashtable with keys and values associated with
     * blackberry contactlist attributes.
     *
     * @param String: contactString
     * @return Hashtable
     */
    private Hashtable buildMapFromXML(String contactString) {
        int startMarkerIndex = contactString.indexOf(START_MARKER);
        int endMarkerIndex = contactString.indexOf(END_MARKER);

        if(startMarkerIndex == -1 || endMarkerIndex == -1) {
            Dialog.inform("Improper data from the server.");
            return null;
        }

        Hashtable contactMap   = new Hashtable();
        String startTag        = "";
        String endTag          = "";
        String degeneratedTag  = "";
        String data            = "";

        for (int i = 0; i < tagArray.length; i++) {
            String tag = tagArray[i];

            startTag = "<"  +tag + ">";

            int startTagIndex = contactString.indexOf(startTag);
            if(startTagIndex == -1) {
                continue;
            }

            degeneratedTag = "<" + tag +"/>";
            int degeneratedTagIndex = contactString.indexOf(degeneratedTag);
            if(degeneratedTagIndex != -1) {
                continue;
            }

            endTag = "</"+tag+">";

            int endTagIndex = contactString.indexOf(endTag);
            if(endTagIndex != -1) {
                data = contactString.substring(startTagIndex+startTag.length(), endTagIndex);
            } else {
                data = "";
            }

            if (data.length() > 0) {
                data = StringTools.unescapeXml(data);
            }

            contactMap.put(tag, data);
        }
        return contactMap;
    }

    
    /**
     * check OS version
     * @param handled OS
     * @return true if O.S. >= 4.0.2
     */
    private boolean checkOS(String os) {
              
        if (os == null || os.length() == 0) {
            StaticDataHelper sdh = new StaticDataHelper();
            sdh.setOS();
            os = sdh.getOS();
        }   
        
        if (os == null || !(os.length() > 3)) {
            return false;
        }    
        
        int major =  Integer.valueOf(os.substring(0,1)).intValue();
        int middle = Integer.valueOf(os.substring(2,3)).intValue();
        
        int minor = 0;
              
        if (os.length() > 4) {
            minor  = Integer.valueOf(os.substring(4,5)).intValue();
        } 
                
        if ((major > 4) || (major == 4 && ((middle == 0 && minor >= 2) || middle >= 1))) {
            return true;
        } else {
            return false;
        }   

    }

}
