/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.syncclient.blackberry.parser;

import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.TimeZone;
import java.util.Vector;

import javax.microedition.pim.Event;
import javax.microedition.pim.EventList;

import net.rim.blackberry.api.pdap.BlackBerryEvent;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.i18n.SimpleDateFormat;

import sync4j.syncclient.common.StringTools;

import sync4j.syncclient.util.StaticDataHelper;

import sync4j.syncclient.util.StaticDataHelper;

/**
 * Provides with the user interface
 * for sync events with SyncServer.
 *
 * @author: Fabio Maggi @ Funambol
 * $Id: XMLEventParser.java,v 1.0
 */
public class XMLEventParser implements EventParser{

    //------------------------------------------Private Data Constants

    private static final String SUMMARY          = "Subject"                    ;
    private static final String LOCATION         = "Location"                   ;
    private static final String NOTE             = "Body"                       ;
    private static final String START            = "Start"                      ;
    private static final String END              = "End"                        ;
    private static final String REVISION         = "Revision"                   ;
    private static final String ALARM            = "ReminderMinutesBeforeStart" ;
    private static final String ALARM_ENABLE     = "ReminderSet"                ;
    private static final String CLASS            = "Class"                      ;
    private static final String ALLDAY           = "AllDayEvent"                ;

    private static final String START_MARKER     = "<appointment>"              ;
    private static final String END_MARKER       = "</appointment>"             ;

    private static final String DATETIME_FORMAT_UTC
                                                 = "yyyyMMdd'T'HHmmss'Z'"       ;

    private static final String DATETIME_FORMAT_ALLDAY
                                                 = "yyyy-MM-dd"                 ;

    private static final String UTC_TIMEZONE     = "UTC"                        ;

    
    //------------------------------------------Private Data menmbers

    private String[] tagArray = {
            ALLDAY       ,
            SUMMARY      ,
            LOCATION     ,
            NOTE         ,
            START        ,
            END          ,
            REVISION     ,
            ALARM        ,
            ALARM_ENABLE ,
            CLASS
    };

    private EventList        list   ;
    private BlackBerryEvent  event  ;
    private boolean          modify ;

    /**
     * This is an initialization constructor for XMLEventParser class.
     *
     * @param EventList: Blackberry event list
     * @param Event: Blackberry event
     * @param boolean: to modify or not to modify
     * @return XMLEventParser
     */
    public XMLEventParser(EventList list, Event event, boolean modify) {

        this.list   = list   ;
        this.event  = (BlackBerryEvent) event  ;
        this.modify = modify ;
    }

    /**
     * This method is invoked to obtain Event object from a event String.
     * if the modify flag is set to true it modifies the event.
     * if the modify flag is set to false it simply adds a event to blackberry calendar.
     *
     * @param String:String containing event information to be parsed.
     * @return Event: that has been added newly/modified in blackberry calendar.
     */
    public Event parseEvent(String eventString) {

        if(modify) {
            return modifyEvent (eventString);
        } else {
            return addEvent    (eventString);
        }

    }

    /**
     * This method is invoked to obtain Event object from a event String,
     * when a event is added to the blackberry event.This method
     * adds a new event and returns the reference of that object.
     *
     * @param String: String containing event information to be parsed.
     * @return Event: Event object that has been added newly.
     */
    public Event addEvent(String eventString) {

        boolean allday = false;

        Hashtable eventMap = buildMapFromXML(eventString);
        if (eventMap != null) {
            try {

                String field = getValue(eventMap, ALLDAY);

                if ("1".equals(field)) {
                    allday = true;
                }

                field = getValue(eventMap, SUMMARY);
                if (isSupportedField(BlackBerryEvent.SUMMARY, field)) {
                    event.addString(BlackBerryEvent.SUMMARY,BlackBerryEvent.ATTR_NONE, field);
                }

                field = getValue(eventMap, LOCATION);
                if (isSupportedField(BlackBerryEvent.LOCATION, field)) {
                    event.addString(BlackBerryEvent.LOCATION, BlackBerryEvent.ATTR_NONE, field);
                }

                field = getValue(eventMap, NOTE);
                if (isSupportedField(BlackBerryEvent.NOTE, field)) {
                    event.addString(BlackBerryEvent.NOTE, BlackBerryEvent.ATTR_NONE, field);
                }

                field = getValue(eventMap, START);
                if (field != null && field.length() > 0) {
                    long dateStart;
                    if (allday) {
                       dateStart = getAlldayDateStart(field);
                    } else {
                       dateStart = getDate(field);
                    }
                    if (isSupportedField(BlackBerryEvent.START, field)) {
                        event.addDate(BlackBerryEvent.START, Event.ATTR_NONE, dateStart);
                    }
                }

                field = getValue(eventMap, END);
                if (field != null && field.length() > 0) {
                    long dateEnd;
                    if (allday) {
                       dateEnd = getAlldayDateEnd(field);
                    } else {
                       dateEnd = getDate(field);
                    }
                    if (isSupportedField(BlackBerryEvent.END, field)) {
                        event.addDate(BlackBerryEvent.END, BlackBerryEvent.ATTR_NONE, dateEnd);
                    }
                }

                if (isSupportedField(BlackBerryEvent.ALLDAY, field)) {
                    if (allday) {
                          event.addBoolean(BlackBerryEvent.ALLDAY, BlackBerryEvent.ATTR_NONE, true);
                    } else {
                         event.addBoolean(BlackBerryEvent.ALLDAY, BlackBerryEvent.ATTR_NONE, false);
                    }
                }

                field = getValue(eventMap, REVISION);
                if (field != null && field.length() > 0) {
                    long dateRevision = getDate(field);
                    if (isSupportedField(BlackBerryEvent.REVISION, field)) {
                        event.addDate(BlackBerryEvent.REVISION, BlackBerryEvent.ATTR_NONE, dateRevision);
                    }
                }

                String alarmEnable = getValue(eventMap, ALARM_ENABLE);

                if ("1".equals(alarmEnable)) {
                    field = getValue(eventMap, ALARM);
                    if (field != null && field.length() > 0 && isSupportedField(BlackBerryEvent.ALARM, field)) {
                        event.addInt(BlackBerryEvent.ALARM, BlackBerryEvent.ATTR_NONE, (Integer.parseInt(field) * 60));
                    }
                }

                field = getValue(eventMap, CLASS);
                if (isSupportedField(BlackBerryEvent.CLASS, field)) {
                    event.addInt(BlackBerryEvent.CLASS, BlackBerryEvent.ATTR_NONE, Integer.parseInt(field));
                }

                return event;

            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }

        return null;
    }

    /**
     * This method is invoked to obtain Event object from a event String,
     * when a event is modified in the blackberry calendar. This method
     * modifies the object and returns the reference to that object.
     *
     * @param String: String containing event information to be parsed.
     * @return Event:Event that has been modified.
     */
    public Event modifyEvent(String eventString) {

        Hashtable eventMap = buildMapFromXML(eventString);

        boolean allday = false;

        if (eventMap != null) {

            try {

                String field = getValue(eventMap, ALLDAY);

                if ("1".equals(field)) {
                    allday = true;
                }

                field = getValue(eventMap, SUMMARY);
                if (isSupportedField(BlackBerryEvent.SUMMARY, field) && event.countValues(BlackBerryEvent.SUMMARY) > 0) {
                        event.setString(BlackBerryEvent.SUMMARY, 0, BlackBerryEvent.ATTR_NONE, field);
                 } else {
                        event.removeValue(BlackBerryEvent.SUMMARY, 0);
                        event.addString(BlackBerryEvent.SUMMARY, Event.ATTR_NONE, field);
                }

                field = getValue(eventMap, LOCATION);
                if (isSupportedField(BlackBerryEvent.LOCATION, field) && event.countValues(BlackBerryEvent.LOCATION) > 0) {
                        event.setString(BlackBerryEvent.LOCATION, 0, Event.ATTR_NONE, field);
                } else {
                        event.addString(BlackBerryEvent.LOCATION, Event.ATTR_NONE, field);
                }

                field = getValue(eventMap, NOTE);

                if (isSupportedField(BlackBerryEvent.NOTE, field) && event.countValues(BlackBerryEvent.NOTE) > 0) {
                        event.setString(BlackBerryEvent.NOTE, 0, Event.ATTR_NONE, field);
                } else {
                        event.addString(BlackBerryEvent.NOTE, Event.ATTR_NONE, field);
                }

                field = getValue(eventMap, START);
                if (field != null && field.length() > 0) {
                    long dateStart;
                    if (allday) {
                       dateStart = getAlldayDateStart(field);
                    } else {
                       dateStart = getDate(field);
                    }
                    event.removeValue(BlackBerryEvent.START, 0);
                    event.addDate(BlackBerryEvent.START, Event.ATTR_NONE, dateStart);
                }

                field = getValue(eventMap, END);
                if (field != null && field.length() > 0) {
                    long dateEnd;
                    if (allday) {
                       dateEnd = getAlldayDateEnd(field);
                    } else {
                       dateEnd = getDate(field);
                    }
                    event.removeValue(BlackBerryEvent.END, 0);
                    event.addDate(BlackBerryEvent.END, Event.ATTR_NONE, dateEnd);
                }
           
                if (isSupportedField(BlackBerryEvent.ALLDAY, field)) {
                    if (allday) {
                          if (isSupportedField(BlackBerryEvent.ALLDAY, field) && event.countValues(BlackBerryEvent.ALLDAY) > 0) {
                            event.removeValue(BlackBerryEvent.ALLDAY, 0);
                            event.addBoolean(BlackBerryEvent.ALLDAY, BlackBerryEvent.ATTR_NONE, true);
                          }
                          
                    } else {
                          if (isSupportedField(BlackBerryEvent.ALLDAY, field) && event.countValues(BlackBerryEvent.ALLDAY) > 0) {
                            event.removeValue(BlackBerryEvent.ALLDAY, 0);
                            event.addBoolean(BlackBerryEvent.ALLDAY, BlackBerryEvent.ATTR_NONE, false);
                          }   
                    }
                }
                              
                field = getValue(eventMap, REVISION);
                if (field != null && field.length() > 0) {
                    long dateRevision = getDate(field);
                    if (isSupportedAtrributedField(BlackBerryEvent.END, Event.ATTR_NONE, field)) {
                            event.removeValue(BlackBerryEvent.REVISION, 0);
                            event.addDate(BlackBerryEvent.REVISION, Event.ATTR_NONE, dateRevision);
                    }
                }

                String alarmEnable = getValue(eventMap, ALARM_ENABLE);

                if ("1".equals(alarmEnable)) {
                    field = getValue(eventMap, ALARM);
                    if (field != null && field.length() > 0) {

                        int alarmSecondsBefore = getAlarmSecondsBefore(field);
                        if (isSupportedField(BlackBerryEvent.ALARM, field) && event.countValues(BlackBerryEvent.ALARM) > 0) {
                                event.setInt(BlackBerryEvent.ALARM, 0, Event.ATTR_NONE, (alarmSecondsBefore * 60));
                        } else {
                                event.addInt(BlackBerryEvent.ALARM, Event.ATTR_NONE, (alarmSecondsBefore * 60));
                        }

                    }

                } else {

                    field = getValue(eventMap, ALARM);
                    if (field != null && field.length() > 0) {

                        int alarmSecondsBefore = getAlarmSecondsBefore(field);
                        if (isSupportedAtrributedField(BlackBerryEvent.ALARM, Event.ATTR_NONE, field) && event.countValues(BlackBerryEvent.ALARM) > 0) {
                                event.removeValue(BlackBerryEvent.ALARM, 0);
                        }

                    }

                }

                field = getValue(eventMap, CLASS);
                if (isSupportedAtrributedField(BlackBerryEvent.CLASS, Event.ATTR_NONE, field)) {
                        event.removeValue(BlackBerryEvent.ALARM, 0);
                        event.addInt(BlackBerryEvent.CLASS, Event.ATTR_NONE, Integer.parseInt(field));
                }

                return event;
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }
        return null;
    }

    /**
     * This method appends all the event information passed to it
     * in the form of event object.
     *
     * @param Event: Blackberry event.
     * @return String: all event information in a string format.
     */
    public String toString(Event event) {

        StringBuffer eventBuffer = new StringBuffer();

        eventBuffer.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");

        eventBuffer.append("<appointment>\n");

        boolean allday = false;

        try {
            allday = ((BlackBerryEvent) event).getBoolean(BlackBerryEvent.ALLDAY, 0);
        } catch (Exception e) {
            StaticDataHelper.log("Not set field: " + ALLDAY);
        }

        appendField           (eventBuffer                ,
                               (BlackBerryEvent) event    ,
                               BlackBerryEvent.SUMMARY    ,
                               SUMMARY                    );

        appendField           (eventBuffer                ,
                               (BlackBerryEvent) event    ,
                               BlackBerryEvent.LOCATION   ,
                               LOCATION                   );

        appendField           (eventBuffer               ,
                               (BlackBerryEvent) event   ,
                               BlackBerryEvent.NOTE      ,
                               NOTE                      );

        appendFieldDate       (eventBuffer              ,
                               (BlackBerryEvent) event  ,
                               BlackBerryEvent.START    ,
                               START                    ,
                               allday                   ,
                               false                    );

        appendFieldBoolean    (eventBuffer              ,
                               (BlackBerryEvent) event  ,
                               BlackBerryEvent.ALLDAY   ,
                               ALLDAY                   );

        appendFieldDate       (eventBuffer               ,
                               (BlackBerryEvent) event   ,
                               BlackBerryEvent.END       ,
                               END                       ,
                               allday                    ,
                               true                      );

        appendFieldAlarmMB    (eventBuffer              ,
                               (BlackBerryEvent) event  ,
                               BlackBerryEvent.ALARM    ,
                               ALARM                    );

        eventBuffer.append("</appointment>");

        return eventBuffer.toString();
    }

    //-----------------------------------------Private Methods

    /**
     * This method appends the event information String Type
     * supported by blackberry to a
     * string buffer.
     *
     * @param StringBuffer: StringBuffer containing event information.
     * @param Event: Blackberry event
     * @param int: type of field
     * @param String: Name of field.
     * @return void.
     */
    private void appendField(StringBuffer eventBuffer ,
                             Event        event       ,
                             int          field       ,
                             String       tag         ) {

        String value = null;

        if (list.isSupportedField(field)) {

            value = "";

            try {
                value = event.getString(field, 0);
            } catch (Exception e) {
                StaticDataHelper.log("Not set field: " + tag);
            }

            appendToEvent(eventBuffer, tag, value);
        }
    }

     /**
     * This method appends the event information String Type
     * supported by blackberry to a
     * string buffer.
     *
     * @param StringBuffer: StringBuffer containing event information.
     * @param Event: Blackberry event
     * @param int: type of field
     * @param String: Name of field.
     * @return void.
     */
    private void appendFieldBoolean(StringBuffer eventBuffer ,
                                    Event        event       ,
                                    int          field       ,
                                    String       tag         ) {

        boolean value = false;

        if (list.isSupportedField(field)) {

            try {
                value = event.getBoolean(field, 0);
            } catch (Exception e) {
                StaticDataHelper.log("Not set field: " + tag);
            }

            if (value) {
                appendToEvent(eventBuffer, tag, "1");
            } else {
                appendToEvent(eventBuffer, tag, "0");
            }
        }
    }

    /**
     * This method appends the event information Date Type
     * supported by blackberry to a
     * string buffer.
     *
     * @param StringBuffer: StringBuffer containing event information.
     * @param Event: Blackberry event
     * @param int: type of field
     * @param String: Name of field.
     * @return void.
     */
    private void appendFieldDate(StringBuffer eventBuffer ,
                                 Event        event       ,
                                 int          field       ,
                                 String       tag         ,
                                 boolean      allday      ,
                                 boolean      endDate     )  {

        StaticDataHelper  sdh            = null ;

        long              value          = 0    ;
        Calendar          rightNow       = null ;
        String            formattedDate  = null ;
        SimpleDateFormat  formatter      = null ;

        String            format         = null ;

        String            osVersion      = null ;

        long              offset         = 0    ;

        sdh = new StaticDataHelper();

        osVersion = sdh.getOS();

        if (list.isSupportedField(field)) {

            try {
                value = event.getDate(field, 0);
            } catch (Exception e) {
                StaticDataHelper.log("Not set field: " + tag);
            }

            if (allday) {
                format = DATETIME_FORMAT_ALLDAY;
                if (endDate) {
                    if (checkOS(osVersion)) {
                         offset = 24*60*60*1000 + getTimeZoneOffSet(new Date(value));
                    } else {
                         offset = getTimeZoneOffSet(new Date(value));
                    }
                } else {
                    offset = getTimeZoneOffSet(new Date(value));
                }
            } else {
                format = DATETIME_FORMAT_UTC;
                offset = getTimeZoneOffSet(new Date(value));
            }

            rightNow = Calendar.getInstance();

            rightNow.setTime(new Date(value));

            formatter      = new SimpleDateFormat (format);

            formattedDate  = formatter.format
                                (new Date(rightNow.getTime().getTime() - offset));

            appendToEvent(eventBuffer, tag, formattedDate);
        }
    }

    /**
     * This method appends the event information int Type
     * supported by blackberry to a
     * string buffer.
     *
     * @param StringBuffer: StringBuffer containing event information.
     * @param Event: Blackberry event
     * @param int: type of field
     * @param String: Name of field.
     * @return void.
     */
    private void appendFieldInt(StringBuffer eventBuffer ,
                                Event        event       ,
                                int          field       ,
                                String       tag         ) {

        String value = null ;

        if (list.isSupportedField(field)) {

            value = "";

            try {
                value = String.valueOf(event.getInt(field, 0));
            } catch (Exception e) {
                StaticDataHelper.log("Not set field: " + tag);
            }

            appendToEvent(eventBuffer, tag, value);
        }
    }

    /**
     * This method appends the event information int Type supported by blackberry to a
     * string buffer.
     *
     * @param StringBuffer: StringBuffer containing event information.
     * @param Event: Blackberry event
     * @param int: type of field
     * @param String: Name of field.
     * @return void.
     */
    private void appendFieldAlarmMB(StringBuffer eventBuffer ,
                                    BlackBerryEvent        event       ,
                                    int          field       ,
                                    String       tag         ) {

        String  value = null ;

        if (list.isSupportedField(field)) {

            int reminder = 0;

            value = "";

            try {
                 reminder = (event.getInt(field, 0))/60;
                 value = String.valueOf(reminder);
             } catch (Exception e) {
                StaticDataHelper.log("Not set field: " + tag);
            }

            appendToEvent(eventBuffer, tag, value);

            if (reminder > 0) {
                appendToEvent(eventBuffer, ALARM_ENABLE, "1");
            } else {
                appendToEvent(eventBuffer, ALARM_ENABLE, "0");
            }

        }
    }

    /**
     * This method appends the event information supported by blackberry to a
     * string buffer.
     *
     * @param StringBuffer: StringBuffer containing event information.
     * @param String: name of tag associated with event information.
     * @param String: value corresponding to tag.
     * @return void
     */
    private void appendToEvent(StringBuffer eventBuffer, String tag, String value) {

        value = (value == null) ? "" : value;

        if (value.length() > 0) {
            value = StringTools.escapeXml(value);
        }

        eventBuffer.append("<").append(tag).append(">").append(value);

        eventBuffer.append("</").append(tag).append(">\n");

    }

    /**
     * This method returns the value associated with a key in a hashtable
     * containing eventMap.
     *
     * @param Hashtable: eventMap
     * @param String: key to which associated value it to be found.
     * @return String: Value associated with the key
     */
    private String getValue(Hashtable eventMap, String key) {
        Object val = eventMap.get(key);
        String value = val == null ? "" : val.toString();
        return value.trim();
    }

    /**
     * This method checks if the field and attibute are supported by the event list
     * of blackberry address book
     *
     * @param int: field
     * @param int: attribute
     * @param String: value
     * @return boolean:returns true if both the field and attribute
     * associated with it r supported
     */
    private boolean isSupportedAtrributedField(int field, int attribute, String value) {
        return isSupportedField(field, value) && list.isSupportedAttribute(field, attribute);
    }

    /**
     * This method checks if the type of field is supported by the event list
     * of blackberry address book.
     *
     * @param int: type
     * @param String: value
     * @return boolean:returns true if event list supports this field
     */
    private boolean isSupportedField(int type, String value) {
        return list.isSupportedField(type);
    }

    /**
     * This method creates a hashtable with keys and values associated with
     * blackberry eventlist attributes.
     *
     * @param String: eventString
     * @return Hashtable
     */
    private Hashtable buildMapFromXML(String eventString) {
        int startMarkerIndex = eventString.indexOf(START_MARKER);
        int endMarkerIndex = eventString.indexOf(END_MARKER);

        if(startMarkerIndex == -1 || endMarkerIndex == -1) {
            Dialog.inform("Improper data from the server.");
            return null;
        }

        Hashtable eventMap   = new Hashtable();
        String startTag        = "";
        String endTag          = "";
        String degeneratedTag  = "";
        String data            = "";

        for (int i = 0; i < tagArray.length; i++) {
            String tag = tagArray[i];

            startTag = "<"+tag+">";

            int startTagIndex = eventString.indexOf(startTag);
            if(startTagIndex == -1) {
                continue;
            }

            degeneratedTag = "<"+tag+"/>";
            int degeneratedTagIndex = eventString.indexOf(degeneratedTag);
            if(degeneratedTagIndex != -1) {
                continue;
            }

            endTag = "</"+tag+">";

            int endTagIndex = eventString.indexOf(endTag);
            if(endTagIndex != -1) {
                data = eventString.substring(startTagIndex+startTag.length(), endTagIndex);
            } else {
                data = "";
            }

            if (data.length() > 0) {
                data = StringTools.unescapeXml(data);
            }

            eventMap.put(tag, data);
        }
        return eventMap;
    }

    /**
     * Get time from date in "yyyyMMddTHHmmssZ" String format
     * @param field date in "yyyyMMddTHHmmssZ" String format
     * @return time from date in "yyyyMMddTHHmmssZ" String format
     */
    private long getDate(String field) {

        int  day    = 0 ;
        int  mounth = 0 ;
        int  year   = 0 ;
        int  hour   = 0 ;
        int  minute = 0 ;
        int  second = 0 ;

        long offset = 0 ;

        Calendar date  = null ;

        day    = Integer.parseInt (field.substring( 6  , 8  ));
        mounth = Integer.parseInt (field.substring( 4  , 6  ));
        year   = Integer.parseInt (field.substring( 0  , 4  ));
        hour   = Integer.parseInt (field.substring( 9  , 11 ));
        minute = Integer.parseInt (field.substring( 11 , 13 ));
        second = Integer.parseInt (field.substring( 13 , 15 ));

        date = Calendar.getInstance();

        date.set(Calendar.DAY_OF_MONTH , day        ) ;
        date.set(Calendar.MONTH        , mounth - 1 ) ;
        date.set(Calendar.YEAR         , year       ) ;
        date.set(Calendar.HOUR_OF_DAY  , hour       ) ;
        date.set(Calendar.MINUTE       , minute     ) ;
        date.set(Calendar.SECOND       , second     ) ;

        return date.getTime().getTime() + getTimeZoneOffSet(date.getTime());

    }

    /**
     * Get time from date in "yyyy-MM-dd" String format
     * @param field date in "yyyy-MM-dd" String format
     * @return time at 00:00:00 from date
     */
    private long getAlldayDateStart(String field) {

        int  day    = 0 ;
        int  mounth = 0 ;
        int  year   = 0 ;

        Calendar date  = null ;

        day    = Integer.parseInt (field.substring( 8  , 10  ));

        mounth = Integer.parseInt (field.substring( 5  , 7   ));
        year   = Integer.parseInt (field.substring( 0  , 4   ));

        date = Calendar.getInstance(TimeZone.getTimeZone("GMT"));

        date.set(Calendar.DAY_OF_MONTH , day        ) ;
        date.set(Calendar.MONTH        , mounth - 1 ) ;
        date.set(Calendar.YEAR         , year       ) ;
        date.set(Calendar.HOUR_OF_DAY  , 0          ) ;
        date.set(Calendar.MINUTE       , 0          ) ;
        date.set(Calendar.SECOND       , 0          ) ;

        return date.getTime().getTime();

    }

    /**
     * Get time from date in "yyyy-MM-dd" String format
     * @param field date in "yyyy-MM-dd" String format
     * @return time at 23:59:59 from date
     */
    private long getAlldayDateEnd(String field) {

        int  day    = 0 ;
        int  mounth = 0 ;
        int  year   = 0 ;

        int  offset = 0;

        Calendar date  = null ;

        StaticDataHelper sdh = new StaticDataHelper();

        String osVersion = sdh.getOS();

        
        offset =  24 * 60 * 60 * 1000;
       
        day    = Integer.parseInt (field.substring( 8  , 10  ));
        mounth = Integer.parseInt (field.substring( 5  , 7   ));
        year   = Integer.parseInt (field.substring( 0  , 4   ));

        date = Calendar.getInstance(TimeZone.getTimeZone("GMT"));

        date.set(Calendar.DAY_OF_MONTH , day        ) ;
        date.set(Calendar.MONTH        , mounth - 1 ) ;
        date.set(Calendar.YEAR         , year       ) ;
        date.set(Calendar.HOUR_OF_DAY  , 00         ) ;
        date.set(Calendar.MINUTE       , 00         ) ;
        date.set(Calendar.SECOND       , 00         ) ;
       
        return date.getTime().getTime() + offset;

    }

    /**
     * return seconds from minutes in String format
     * @param field minutes in String format
     * @return seconds
     */
    private int getAlarmSecondsBefore(String field)
    throws NumberFormatException {
        int alarmMinutesBefore = Integer.parseInt(field);
        return alarmMinutesBefore;
    }

    /**
     * @param date date to read offset
     * return timezone offset
     */
    private long getTimeZoneOffSet(Date date) {

        Calendar now = null;
        TimeZone zn = null;
        long offset = 0;

        now = Calendar.getInstance();

        now.setTime(date);

        zn = TimeZone.getDefault();

        offset = zn.getOffset(1                                              ,
                              now.get(Calendar.YEAR)                         ,
                              now.get(Calendar.MONTH)                        ,
                              now.get(Calendar.DAY_OF_MONTH)                 ,
                              now.get(Calendar.DAY_OF_WEEK)                  ,
                              now.get(Calendar.HOUR_OF_DAY) * 60 * 60 * 1000 +
                              now.get(Calendar.MINUTE) * 60 * 1000           +
                              now.get(Calendar.SECOND) * 1000                );


        return offset;

    }

    /**
     * check OS version
     * @param handled OS
     * @return true if O.S. >= 4.1.0
     */
    private boolean checkOS(String os) {

        int major =  Integer.valueOf(os.substring(0,1)).intValue();
        int middle = Integer.valueOf(os.substring(2,3)).intValue();

        int minor = 0;

        if (os.length() > 4) {
            minor  = Integer.valueOf(os.substring(4,5)).intValue();
        }

        if ((major > 4) || (major == 4 && middle >= 1)) {
            return true;
        } else {
            return false;
        }

    }

}
