/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.syncclient.spds;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import javax.microedition.io.HttpConnection;
import javax.microedition.io.Connector;

import sync4j.syncclient.util.StaticDataHelper;

import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.system.RadioInfo;

/**
 *  Rapresents a HTTP client implementation
 *
 *  @author Fabio Maggi @ Funambol
 *  @version $Id: SyncMLClientImpl.java,v 1.7 2006/02/07 09:47:17 luigiafassina Exp $
 *
 **/
public final class SyncMLClientImpl {

    // ----------------------------------------------------------------------------------------- Constants

    private static final String PROP_MICROEDITION_CONFIGURATION = "microedition.configuration"  ;
    private static final String PROP_CONTENT_LANGUAGE           = "Content-Language"            ;
    private static final String PROP_CONTENT_LENGTH             = "Content-Length"              ;
    private static final String PROP_CONTENT_TYPE               = "Content-Type"                ;
    private static final String PROP_MICROEDITION_LOCALE        = "microedition.locale"         ;
    private static final String PROP_MICROEDITION_PROFILES      = "microedition.profiles"       ;
    private static final String PROP_USER_AGENT                 = "User-Agent"                  ;

    // ----------------------------------------------------------------------------------------- Private Data

    private final String requestURL;

    // ------------------------------------------------------------ Constructors
    /**
     *
     * @param requestURL must be non-null
     *
     */
    public SyncMLClientImpl(final String requestURL) throws IOException
    {
        if (requestURL == null)
        {
            throw new NullPointerException("request URL parameter is null");
        }

        this.requestURL = requestURL;
    }


    // ---------------------------------------------------------- Public methods

    /**
     *
     * @param request HTTP request
     * @return HTTP response
     */
    public String sendMessage(final String request)
    throws IOException, NetworkException
    {
        HttpConnection   httpCon      = null ;
        DataInputStream  inputStream  = null ;
        DataOutputStream outputStream = null ;
    
        StringBuffer b = null;
        byte[] data    = null;     
        
        synchronized(this) {
        
        boolean signal = true;
            
        if ((RadioInfo.getState() != RadioInfo.STATE_ON) ||
            (RadioInfo.getSignalLevel() == RadioInfo.LEVEL_NO_COVERAGE)) {
            signal = false;
        }
        
        if (!signal) {
            throw new NetworkException();
        }
       
        try {
      
            StaticDataHelper sdh = new StaticDataHelper();
            
            byte[] dataToSend =  request.getBytes();
                   
            httpCon = (HttpConnection)Connector.open(requestURL);
            
            httpCon.setRequestMethod   (HttpConnection.POST               );
            httpCon.setRequestProperty (PROP_USER_AGENT     , 
                                        "BlackBerry/" + sdh.getOS()       );
            httpCon.setRequestProperty (PROP_CONTENT_LENGTH , 
                                        String.valueOf(dataToSend.length) );
            httpCon.setRequestProperty (PROP_CONTENT_TYPE   , 
                                        "application/vnd.syncml+xml"      );
        
            String locale  = System.getProperty(PROP_MICROEDITION_LOCALE  );
            
            if (locale != null) {
                httpCon.setRequestProperty(PROP_CONTENT_LANGUAGE, locale  );
            }
            
            outputStream = httpCon.openDataOutputStream();  
            outputStream.write(dataToSend);
            outputStream.flush();
                               
            inputStream = httpCon.openDataInputStream();
            
            int size = (int) httpCon.getLength();        
     
            //len = c.getLength();
            b = new StringBuffer(size >= 0 ? (int)size : 4096);
            if (size != -1) {
                // Read content-Length bytes, or until max is reached.
                int ch = 0;
                for (int i = 0; i < size; i++) {
                    if ((ch = inputStream.read()) != -1) {
                        if (ch <= ' ') {
                            ch = ' ';
                        }
                        b.append((char) ch);
                    }
                }
            } else {
                // Read til the connection is closed, or til max is reached.
                // (Typical HTTP/1.0 script generated output)
                int ch = 0;
                size = 0;
                while ((ch = inputStream.read()) != -1) {
                    if (ch <= ' ') {
                        ch = ' ';
                    }
                    b.append((char)ch);
                }
            }
    
            data = b.toString().getBytes();

            } finally {
                if (inputStream != null)  {
                    inputStream.close();
                    inputStream = null ;
                }
                if (outputStream != null) {
                    outputStream.close();
                    outputStream = null ;
                }
                if (httpCon != null)      {
                    httpCon.close();
                    httpCon = null ;
                }
    
            }
        }
        
        return new String(data);
    }

    // ---------------------------------------------------------- Private metho

}
