/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package sync4j.syncclient.sps;

import java.util.Enumeration;
import java.util.Vector;

import javax.microedition.rms.RecordStore;

import javax.microedition.pim.PIM;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.util.StringMatch;
import sync4j.syncclient.util.StaticDataHelper;

import sync4j.syncclient.blackberry.parser.ParserFactory;
import sync4j.syncclient.blackberry.parser.EventParser;

/**
 * This class provide abstract methods
 * to access device database
 *
 * @version $Id: DataStore.java,v 1.10 2006/02/07 09:47:17 luigiafassina Exp $
 */
public abstract class DataStore {
      //------------------------------------------------------------- Constants

    public static final char   RECORD_STATE_NEW      = 'N'            ;
    public static final char   RECORD_STATE_DELETED  = 'D'            ;
    public static final char   RECORD_STATE_UPDATED  = 'U'            ;
    public static final char   RECORD_STATE_UNSIGNED = ' '            ;

    protected static final long PERSISTENCE_KEY = 0x50b137116d9be33cL ;

    public  static final int    MAX_ITEM_NUMBER = 24                  ;


    //------------------------------------------------------------- Private Data

    protected PIMList pimList;
    protected int page;
    protected Enumeration items;

    // ------------------------------------------------------------------------

    public static DataStore getDataStore (String datastoreName) {

        StaticDataHelper stdh = new StaticDataHelper();

        if (stdh.getContactSourceUri().equals(datastoreName))   {
            return new ContactDataStore ();
        } else {
            return new EventDataStore   ();
        }
    }

    protected DataStore() {
        pimList = null;
        page = MAX_ITEM_NUMBER;
    }

    /**
     * Set last timestamp in dedicate recordStore
     * @param lastTimestamp
     * @throws DataAccessException
     **/
    public abstract void setLastTimestamp(long lastTimestamp)
    throws DataAccessException;


    /**
     * @return last timestamp from dedicate recordstore
     * @throws DataAccessException
     **/
    public abstract long getLastTimestamp()
    throws DataAccessException;

    /**
     * if record exist in database, update records
     * if record not exist in database, add record
     * @param record record to store
     * @throws DataAccessException
     **/
    public abstract Record setRecord(Record record, boolean modify)
    throws DataAccessException;

    /**
     * Delete a record from the event database.
     * @param record
     * @throws DataAccessException
     */
    public abstract void deleteRecord(Record record)
    throws DataAccessException;


    /**
     * return no deleted records from device recordstore
     *
     * @return records found
     *
     * @throws DataAccessException
     **/
    public abstract boolean getNextRecords(Vector v) throws DataAccessException;

    /**
     * return record from recordstore
     * filter by record state
     *
     * @return records found
     *
     * @throws DataAccessException
     **/
    public abstract boolean getNextRecords(Vector v, char state) throws DataAccessException;

    public abstract void startDSOperations();

    public abstract void resetModificationCursor();

    /**
     * execute commit recordstore operations
     * remove records signed as DELETED 'D'
     * mark UNSIGNED ' ' records signed as NEW 'N' and UPDATED 'U'
     *
     * @throws DataAccessException
     *
     */
    public abstract void commitDSOperations()
    throws DataAccessException;


    public void setPage(int page) {
        this.page = page;
    }

    public int getPage() {
        return page;
    }
}
