/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.syncclient.util;

import java.util.Vector;

/**
 * This class represents a Vector where the items can be accessed in a paged 
 * way, so that the items can be accessed page per page in once.
 * 
 * 
 */
public class PagedVector extends Vector {

    // --------------------------------------------------------------- Constants
    
    // ------------------------------------------------------------ Private data

    private int page;
    private int nextPageIndex;

    // -------------------------------------------------------------------------

    /**
     *  Create new PagedVector
     *
     *  @param vector input vector
     *  @param indexFrom index from begin new vector
     */
    public PagedVector(int page) {
        this.page = page;
        this.nextPageIndex = 0;
    }
    
    /**
     *  Create new PagedVector
     *
     *  @param vector input vector
     *  @param indexFrom index from begin new vector
     */
    public PagedVector(Vector v, int page) {
        this(page);
        
        for (int i=0, s=v.size(); i<s; ++i) {
            addElement(v.elementAt(i));
        }
    }


    // ---------------------------------------------------------- Public methods
    
    /**
     * Fills the given Vector with the elements of the next page.
     * 
     * @return true if there are more elements, false otherwise
     */
    public boolean getNextPage(final Vector elements) {
        boolean ret = true;

        int s = size();

        elements.removeAllElements(); // just in case...
        for (int i=0; i<page; ++i) {
            if (nextPageIndex>= s) {
                ret = false;
                break;
            }

            elements.addElement(elementAt(nextPageIndex++));
        }

        return ret;
    }

    /**
     * Resets the current position
     *
     */
    public void reset() {
        nextPageIndex = 0;
    }

    /**
     * Are we at the end of the vector?
     *
     * @return true if the last item was returned in the last call to 
     *         getNextPage, false otherwise
     */
    public boolean hasMore;
} 
