/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.syncclient.util;

import java.util.Hashtable;
import java.io.InputStream;
import java.io.IOException;

import net.rim.device.api.system.*;

/**
 * This class supports
 * general using, language dictionary access
 * statics data and static methods
 *
 * @author  Fabio Maggi @ Funambol
 * @version $Id: StaticDataHelper.java,v 1.15 2006/02/07 09:47:17 luigiafassina Exp $
 *
 **/

public class StaticDataHelper {

    //----------------------------------------------------------------- Constants

    final private static String KEY_HOMEPAGE             = "homepage-default"            ;
    final private static String KEY_LOGIN                = "login-default"               ;
    final private static String KEY_CONTACT_SOURCE_URI   = "contact-source-uri-default"  ;
    final private static String KEY_CALENDAR_SOURCE_URI  = "calendar-source-uri-default" ;
    final private static String KEY_GATEWAY_APN          = "gateway-apn-default"         ;
    final private static String KEY_GATEWAY_IP           = "gateway-ip-default"          ;
    final private static String KEY_GATEWAY_PORT         = "gateway-port-default"        ;
    final private static String KEY_SYNC_CONTACT         = "sync-contact-default"        ;
    final private static String KEY_SYNC_CALENDAR        = "sync-calendar-default"       ;
    final private static String KEY_ENCODE               = "encode-default"              ;

    final private static String HOST_PORT_DEFAULT        = "8080"                        ;
    final private static String GATEWAY_PORT_DEFAULT     = "9201"                        ;

    final private static String XML_LANGUAGE             = "/xml/language.xml"           ;

   //----------------------------------------------------------------- Private data

    private   static String                 homePageDefault          = null  ;
    private   static String                 loginDefault             = null  ;
    private   static String                 userDefault              = null  ;
    private   static String                 passwordDefault          = null  ;
    private   static String                 contactSourceUriDefault  = null  ;
    private   static String                 calendarSourceUriDefault = null  ;
    private   static String                 gatewayApnDefault        = null  ;
    private   static String                 gatewayIpDefault         = null  ;
    private   static String                 gatewayPortDefault       = null  ;
    private   static boolean                syncContactDefault       = false ;
    private   static boolean                syncCalendarDefault      = false ;
    private   static boolean                encodeDefault            = false ;

    private   static String                 contactSourceUri         = null  ;
    private   static String                 calendarSourceUri        = null  ;
    private   static String                 operatingSystem          = null  ;
 
    /** debug enable */
    private static boolean   debug        = false ;

    /** language words */
    private static String    language     = null  ;

    /** language words */
    private static Hashtable languageHash = new Hashtable();

    //----------------------------------------------------------------- Public methods

    
    public String getHomePageDefault          () {
        return homePageDefault;
    }

    public String getUserDefault              () {
        return userDefault;
    }

    public String getPasswordDefault          () {
        return passwordDefault;
    }

    public String getContactSourceUriDefault  () {
        return contactSourceUriDefault;
    }

    public String getCalendarSourceUriDefault () {
        return calendarSourceUriDefault;
    }

    public String getGatewayApnDefault        () {
        return gatewayApnDefault;
    }

    public String getGatewayIpDefault         () {
        return gatewayIpDefault;
    }

    public String getGatewayPortDefault       () {
        return gatewayPortDefault;
    }

    public boolean getSyncContactDefault      () {
        return syncContactDefault;
    }

    public boolean getSyncCalendarDefault     () {
        return syncCalendarDefault;
    }

    public String getContactSourceUri         () {
        return contactSourceUri;
    }

    public String getOS                       () {
        return operatingSystem;
    }

    public void setContactSourceUri  (String contactSUri) {
        contactSourceUri = contactSUri;
    }

    public void setCalendarSourceUri (String calendarSUri) {
        calendarSourceUri = calendarSUri;
    }
    
    public void setOS () {
        ApplicationDescriptor[] appDes = ApplicationManager.getApplicationManager().getVisibleApplications();

        String version = "";

        for (int i = appDes.length - 1; i >= 0; --i) {

            if ((appDes[i].getModuleName()).equals("net_rim_bb_ribbon_app")) {
                version =  appDes[i].getVersion();
            }
        }
        operatingSystem = version;
    }

    public boolean isEncode                   () {
        return encodeDefault;
    }

    /**
     * Logs a string.
     *
     * @param msg the string to be logged
     **/
    public static void log(String msg) {
        if (debug)
            System.out.println(msg);
    }

    /**
     * Make a String by value of <i>tag</i>.
     *
     * @param key key to find
     * @return key value
     **/
    public static String getLanguage(String key) {

        String startTag      = null ;
        String endTag        = null ;

        String value         = null ;

        Object valueFromHash = null ;

        valueFromHash = languageHash.get(key);

        if (valueFromHash != null) {
            return (String) valueFromHash;
        }

        startTag = "<"  + key + ">" ;
        endTag   = "</" + key + ">" ;

        String msg = "Parsing XML, TAG " + key ;
        StaticDataHelper.log (msg);

        value = language.substring
                (language.indexOf(startTag) + startTag.length(), language.indexOf(endTag)).trim();

        languageHash.put(key, value);

        return value;

    }

    /**
     * load xml language file
     */
    public static void loadLanguage (Class c)
    throws Exception{
        try {
            language = read(c.getResourceAsStream(XML_LANGUAGE));
        } catch (IOException e ) {

            String msg = "Error: load "         +
                         XML_LANGUAGE           +
                         " file: "              +
                         e.getMessage()         ;

            StaticDataHelper.log (msg);
            throw e;
        }
    }

    /**
     * Reads the content of the given input stream.
     *
     * @param is the input stream
     **/
    public static String read(InputStream is) throws IOException {
        StringBuffer sb = new StringBuffer();

        try {

            byte[] buf = new byte[1024];

            int nbyte = -1;
            while ((nbyte = is.read(buf)) >= 0) {
                sb.append(new String(buf, 0, nbyte));
            }
        } finally {
            is.close();
        }

        return sb.toString();
    }

    /**
     * Load default properties form jad file
     */
    public void loadDefaultValue()
    throws Exception {

        String enableContact  = null ;
        String enableCalendar = null ;
        String encode         = null ;

        homePageDefault          = getLanguage(KEY_HOMEPAGE            );
        loginDefault             = getLanguage(KEY_LOGIN               );
        contactSourceUriDefault  = getLanguage(KEY_CONTACT_SOURCE_URI  );
        calendarSourceUriDefault = getLanguage(KEY_CALENDAR_SOURCE_URI );
        gatewayApnDefault        = getLanguage(KEY_GATEWAY_APN         );
        gatewayIpDefault         = getLanguage(KEY_GATEWAY_IP          );
        gatewayPortDefault       = getLanguage(KEY_GATEWAY_PORT        );
        enableContact            = getLanguage(KEY_SYNC_CONTACT        );
        enableCalendar           = getLanguage(KEY_SYNC_CALENDAR       );
        encode                   = getLanguage(KEY_ENCODE              );

        if (homePageDefault == null && !(homePageDefault.length() > 0)) {
            homePageDefault = "";
        }

        if (loginDefault == null) {
            userDefault     = "" ;
            passwordDefault = "" ;
        } else if ((loginDefault.length() > 0) && loginDefault.indexOf(":") == -1) {
            String msg = "Error: load jad properties wrong login";
            StaticDataHelper.log (msg);
            throw new Exception(msg);
        }

        userDefault     = loginDefault.substring(0, loginDefault.indexOf(":")  );
        passwordDefault = loginDefault.substring(loginDefault.indexOf(":") + 1 );

        if (contactSourceUriDefault == null) {
            contactSourceUriDefault = "";
        }

        if (calendarSourceUriDefault == null) {
            calendarSourceUriDefault = "";
        }

        if (gatewayApnDefault == null) {
            gatewayApnDefault = "";
        }

        if (gatewayIpDefault == null) {
            gatewayIpDefault = "";
        }

        if (gatewayPortDefault == null || !(gatewayPortDefault.length() > 0)) {
            gatewayPortDefault = GATEWAY_PORT_DEFAULT;
        } else {
            try {
                Integer.parseInt(gatewayPortDefault);
            } catch (NumberFormatException e){
                String msg = "Error: load jad properties wrong gateway-port";
                StaticDataHelper.log (msg);
                throw new Exception(msg);
            }
        }

        if ("true".equals(enableContact))  {
            syncContactDefault = true;
        }

        if ("true".equals(enableCalendar)) {
            syncCalendarDefault = true;
        }

        if ("true".equals(encode))         {
            encodeDefault = true;
        }

    }

}
